# -*- coding: utf-8 -*-
"""
Created on Mon Apr  1 12:55:28 2024
 
@author: ma_chenglong
"""
import pandas as pd
from datetime import datetime, timedelta
import os
import numpy as np
 
# 读取fcd文件为DataFrame
def read_fcd_file(file_path):
    with open(file_path, 'r',encoding='utf-8') as f:
        lines = f.readlines()
    return pd.DataFrame([line.strip().split('\t') for line in lines])
 
# 提取并转换时间
def extract_and_convert_time(time_str):
    time_str = time_str.replace('年', '-').replace('月', '-').replace('日', '')
    return datetime.strptime(time_str, "Time: %Y-%m-%d %H:%M:%S")
 
# 转换并处理DataFrame
def convert_and_process(df):
    # 提取时间
    initial_time_str = df.iloc[2, 0]  # 现在时间位于第三行第一列
    initial_time = extract_and_convert_time(initial_time_str)
    # 找到包含'I'的行的索引
    I_indices = df[df[1].fillna('').str.contains('I')].index
    # 由于第二列只存在一行有一个"I"，取得该索引
    if not I_indices.empty:
        idx = I_indices[0]
        # 删除"I"字段之前的所有行，保留"I"字段所在行
        if idx + 1 < len(df):
            df = df.drop(df.index[:idx]).drop(df.index[idx+1])
        else:
            df = df.drop(df.index[:idx])
        # 重置索引
        df = df.reset_index(drop=True)
    # 更新时间
    for i in range(1, len(df)):  # 从第三行开始更新时间
        seconds = float(df.iloc[i, 0])
        new_time = initial_time + timedelta(seconds=seconds)
        df.iloc[i, 0] = new_time.strftime('%Y-%m-%d %H:%M:%S')
    return df
 
# 定义assign_group函数
def assign_group(value, group_width=2):
    """将值分配到基于指定宽度的分组中。"""
    value = float(value)
    return np.floor(value / group_width) * group_width
 
# 修改extract_median_per_grouped函数以包含分组逻辑
def extract_median_per_grouped(df, column_name, sort_by_column, min_rows=5, group_width=3):
    result_df = pd.DataFrame(columns=df.columns)
    # 应用分组函数
    df['Group'] = df[column_name].apply(lambda x: assign_group(x, group_width=group_width))
    # 获取分组后的唯一值
    unique_groups = df['Group'].unique()
    unique_groups.sort()
    for group in unique_groups:
        # 获取当前组的所有行
        group_rows = df[df['Group'] == group]
        # 如果分组内的行数超过min_rows，则对这些行进行处理
        if len(group_rows) > min_rows:
            # 确定后3/4行的开始索引
            start_index = len(group_rows) // 3  # 整除得到1/3的数量，剩余的即后2/3
            # 仅对后3/4的行进行排序
            sorted_rows = group_rows.iloc[start_index:].sort_values(by=sort_by_column)
            # 选择排序后的中间一行
            middle_index = len(sorted_rows) // 2
            middle_row = sorted_rows.iloc[[middle_index]]  # 如果行数是偶数，将选择中间两行中的下一行作为代表行
            # 将选择的中间行添加到结果DataFrame中
            result_df = pd.concat([result_df, middle_row], ignore_index=True)
    # 删除辅助列
    result_df.drop(columns=['Group'], inplace=True)
    return result_df
 
def save_to_csv(df, output_file):
    # 设置第一行第一列和第三列的值
    df.iloc[0, 0] = 'Time'
    df.iloc[0, 2] = 'I (mA/cm²)'
    # 使用第一行的值设置为列名
    df.columns = df.iloc[0]
    # 删除用作列名的那一行
    df = df.drop(df.index[0])
    # 遍历DataFrame，从第2列开始，将所有数字转换为非科学计数法的形式
    for col in df.columns[1:]:  # 从'Time'之后的第一列开始
        df[col] = pd.to_numeric(df[col], errors='coerce')  # 尝试转换为数值
        df[col] = df[col].astype('object')  # 将列转换为object类型
        # 对所有行应用格式化，保留6位小数
        df[col] = df[col].map(lambda x: f'{x:.6f}' if not pd.isnull(x) else '', na_action='ignore')
    # 假设'E_Stack (V)'是我们需要处理的列
    processed_df = extract_median_per_grouped(df, 'I (mA/cm²)','E_Stack (V)')
    # 将处理后的DataFrame保存为CSV
    processed_df.to_csv(output_file, index=False, header=True)
 
# 注意：请根据实际情况调整列名和参数
 
 
# 主函数
def main(fcd_file, output_csv):
    df = read_fcd_file(fcd_file)
    df_processed = convert_and_process(df)
    save_to_csv(df_processed, output_csv)
 
if __name__ == "__main__":
    script_dir = os.path.dirname(os.path.abspath(__file__))
    found_file = False
    for filename in os.listdir(script_dir):
        if (".fcd" in filename) and ("PC" in filename or "ORR" in filename):
            fcd_file = os.path.join(script_dir, filename)
            output_csv = os.path.join(script_dir, filename.replace(".fcd", "-calculated.csv"))
            main(fcd_file, output_csv)
            print(f"转换完成: {fcd_file} -> {output_csv}")
            found_file = True
 
    if not found_file:
        print("未找到符合条件的FCD文件")
